module Conformity
  class Form
    extend Forwardable

    attr_reader :host, :steps

    def_delegators :success_conditions, :has_content?, :has_status_code?,
      :success?
    def_delegators :fields_container, :fields, :field_names

    def initialize(host = '', &steps)
      @host = host
      @steps = steps
      instance_eval(&steps)
    end

    def fields_container
      @fields_container ||= Fields.new
    end

    def actions
      @actions ||= Actions.new(fields_container)
    end

    def success_conditions
      @success_conditions ||= SuccessConditions.new
    end

    def field(name, opts = {})
      fields_container.add_new_field(name, opts)
    end

    def fill
      Capybara.current_session.reset!
      set_host(host)

      actions.instance_eval(&steps)
      Capybara.page
    end

    def fill_with(field_hash)
      fields_container.set_all(field_hash)
      fill
    end

    def fields
      fields_container.fields
    end

    def method_missing(name, *args, &block)
      # ignore actions
      super unless Actions::ACTIONS.include?(name)
    end

    # so Kernel#select isn't called
    def select(*args)
    end

  private
    def set_host(host)
      Capybara.app_host = host
    end
  end
end
